@@ -83,7 +83,7 @@ pub struct Ph {
             fd: libc::c_int,
             flags: libc::c_int,
             /* CALLBACK */
-            callback: Option<unsafe extern "C" fn(fd: libc::c_int, flags: libc::c_int, user_data: *mut libc::c_void) -> libc::c_int>,
+            callback: unsafe extern "C" fn(fd: libc::c_int, flags: libc::c_int, user_data: *mut libc::c_void) -> libc::c_int,
             userdata: *mut libc::c_void) -> *const HexchatHook,
     pub hexchat_unhook: unsafe extern "C" fn(ph: *mut HexchatPlugin,
             hook: *const HexchatHook) -> *const libc::c_void,
@@ -118,7 +118,7 @@ pub struct Ph {
             string: *mut *const libc::c_char,
             integer: *mut libc::c_int) -> libc::c_int,
     pub hexchat_list_get: unsafe extern "C" fn(ph: *mut HexchatPlugin,
-            name: *const libc::c_char) -> *const HexchatList,
+            name: *const libc::c_char) -> *mut HexchatList,
     pub hexchat_list_free: unsafe extern "C" fn(ph: *mut HexchatPlugin,
             xlist: *const HexchatList),
     pub hexchat_list_fields: unsafe extern "C" fn(ph: *mut HexchatPlugin,
