@@ -149,11 +149,11 @@
  *         -[ ] gtkwin_ptr
  *         -[ ] win_ptr
  *     -[x] hexchat_get_prefs
- *     -[ ] hexchat_list_get
- *     -[ ] hexchat_list_fields
- *     -[ ] hexchat_list_next
- *     -[ ] hexchat_list_str
- *     -[ ] hexchat_list_int
+ *     -[x] hexchat_list_get
+ *     -[x] ~~hexchat_list_fields~~ not available. no alternative provided.
+ *     -[x] hexchat_list_next
+ *     -[x] hexchat_list_str
+ *     -[x] hexchat_list_int
  *     -[ ] hexchat_list_time
  *     -[x] ~~hexchat_list_free~~ not available - use Drop impls.
  *     -[x] hexchat_hook_command
@@ -183,27 +183,45 @@
  * -[ ] ???
  */
 
+#![cfg_attr(feature="nightly_tests", feature(c_variadic))]
+
 #[macro_use]
 extern crate impl_trait;
 #[doc(hidden)]
 pub extern crate libc;
 
+// private macros
+
+/// Calls a function on a PluginHandle struct.
+macro_rules! ph_call {
+    ($f:ident($ph:expr, $($args:expr),*)) => {
+        ((*$ph.data.ph).$f)($ph.plugin, $($args),*)
+    };
+    ($f:ident($ph:expr $(,)?)) => {
+        ((*$ph.data.ph).$f)($ph.plugin)
+    };
+}
+
 mod eat;
 mod extra_tests;
 mod infoid;
 mod internals;
+pub mod list;
+#[doc(hidden)]
+#[cfg(feature="nightly_tests")]
+pub mod mock;
 mod pluginfo;
 mod strip;
 mod word;
 
 pub use eat::*;
 pub use infoid::InfoId;
-pub use word::*;
 pub use strip::*;
+pub use word::*;
 
-use pluginfo::PluginInfo;
-use internals::Ph as RawPh;
 use internals::HexchatEventAttrs as RawAttrs;
+use internals::Ph as RawPh;
+use pluginfo::PluginInfo;
 
 use std::borrow::Cow;
 use std::cell::Cell;
@@ -227,18 +245,6 @@ use std::time::{SystemTime, UNIX_EPOCH, Duration};
 #[doc(hidden)]
 pub use libc::{c_char, c_int, c_void, time_t};
 
-// private macros
-
-/// Calls a function on a PluginHandle struct.
-macro_rules! ph_call {
-    ($f:ident($ph:expr, $($args:expr),*)) => {
-        ((*$ph.data.ph).$f)($ph.plugin, $($args),*)
-    };
-    ($f:ident($ph:expr $(,)?)) => {
-        ((*$ph.data.ph).$f)($ph.plugin)
-    };
-}
-
 // ****** //
 // PUBLIC //
 // ****** //
@@ -337,16 +343,21 @@ pub enum PrefValue {
 }
 
 mod valid_context {
+    use std::ptr;
+
+    use crate::list;
     use crate::PluginHandle;
 
-    /// A PluginHandle operating on a valid context.
+    /// A `PluginHandle` operating on a valid context.
     ///
-    /// This mechanism attempts to reduce the likelihood of segfaults in
-    /// hexchat code.
+    /// Methods that require a valid plugin context are only available through
+    /// this. Methods that may invalidate the plugin context also consume this.
     ///
     /// See also [`PluginHandle::ensure_valid_context`].
     pub struct ValidContext<'a, 'ph: 'a> {
-        pub(crate) ph: &'a mut PluginHandle<'ph>,
+        // NOTE: NOT mut(!)
+        pub(crate) ph: &'a PluginHandle<'ph>,
+        fields: list::Fields<'a, 'ph, list::Contexts>,
         _hidden: (),
     }
 
@@ -357,7 +368,27 @@ mod valid_context {
         ///
         /// The PluginHandle's context must be valid.
         pub(crate) unsafe fn new(ph: &'a mut PluginHandle<'ph>) -> Self {
-            Self { ph, _hidden: () }
+            static CONTEXTS: list::Contexts = list::Contexts;
+            Self {
+                ph: &*ph,
+                fields: list::Fields {
+                    context: &*ph,
+                    // this isn't actually documented but hexchat explicitly
+                    // supports passing in NULL for the current context.
+                    // this gives access to some properties that aren't
+                    // available through get_info.
+                    list: ptr::null_mut(),
+                    _t: &CONTEXTS,
+                },
+                _hidden: (),
+            }
+        }
+    }
+
+    impl<'a, 'ph: 'a> std::ops::Deref for ValidContext<'a, 'ph> {
+        type Target = list::Fields<'a, 'ph, list::Contexts>;
+        fn deref(&self) -> &Self::Target {
+            &self.fields
         }
     }
 }
@@ -748,13 +779,8 @@ impl<'ph> PluginHandle<'ph> {
     ///
     /// Returns `true` if the context is valid, `false` otherwise.
     pub fn set_context(&mut self, ctx: &Context<'ph>) -> bool {
-        if let Some(ctx) = ctx.ctx.upgrade() {
-            unsafe {
-                ph_call!(hexchat_set_context(self, *ctx)) != 0
-            }
-        } else {
-            false
-        }
+        // we could've made this &self but we didn't. avoid breaking API.
+        self.set_context_internal(ctx)
     }
 
     /// Do something in a valid context.
@@ -1277,6 +1303,17 @@ impl<'ph> PluginHandle<'ph> {
             ctx.map(|ctx| wrap_context(self, ctx))
         }
     }
+
+    /// Same as `set_context` but it takes `&self`.
+    fn set_context_internal(&self, ctx: &Context<'ph>) -> bool {
+        if let Some(ctx) = ctx.ctx.upgrade() {
+            unsafe {
+                ph_call!(hexchat_set_context(self, *ctx)) != 0
+            }
+        } else {
+            false
+        }
+    }
 }
 
 impl<'a> EventAttrs<'a> {
@@ -1520,6 +1557,36 @@ impl<'a, 'ph: 'a> ValidContext<'a, 'ph> {
         }
     }
 
+    /// Retrieves a list.
+    ///
+    /// # Examples
+    ///
+    /// ```no_run
+    /// use hexchat_unsafe_plugin::list::Contexts;
+    /// use hexchat_unsafe_plugin::PluginHandle;
+    /// 
+    /// fn print_contexts(ph: &mut PluginHandle<'_>) {
+    ///     ph.ensure_valid_context(|ph| {
+    ///         let mut contexts = ph.list(&Contexts);
+    ///         while let Some(context) = contexts.next() {
+    ///             write!(ph, "{}", context.name().unwrap());
+    ///         }
+    ///     })
+    /// }
+    /// ```
+    pub fn list<T: list::List>(&'a self, t: &'a T) -> list::Entries<'a, 'ph, T> {
+        let ph = &self.ph;
+        let list = CString::new(&*t.name()).unwrap();
+        let list = unsafe {
+            ph_call!(hexchat_list_get(ph, list.as_ptr()))
+        };
+        list::Entries {
+            context: self.ph,
+            list: list,
+            t: t,
+        }
+    }
+
     // ******** //
     // FORWARDS //
     // ******** //
@@ -1544,7 +1611,7 @@ impl<'a, 'ph: 'a> ValidContext<'a, 'ph> {
     // context.
     // So either way we're still in a valid context when this returns.
     pub fn set_context(&mut self, ctx: &Context<'ph>) -> bool {
-        self.ph.set_context(ctx)
+        self.ph.set_context_internal(ctx)
     }
 
     /// Prints to the hexchat buffer.
@@ -1856,7 +1923,7 @@ pub unsafe fn hexchat_plugin_init<'ph, T>(plugin_handle: LtPhPtr<'ph>,
             // must register this before the plugin registers anything else!
             let context_hook = pluginhandle.hook_print("Close Context", c_int::min_value(), move |ph, _| {
                 // just remove the context! it's that simple!
-                let ctx = unsafe { ph_call!(hexchat_get_context(ph)) };
+                let ctx = ph_call!(hexchat_get_context(ph));
                 contexts.borrow_mut().remove(&ctx);
                 EAT_NONE
             });
